@@ -1,10 +1,9 @@
 """Define a SimpliSafe account."""
-# pylint: disable=protected-access,too-many-instance-attributes
-
+# pylint: disable=protected-access
 import logging
 from datetime import datetime, timedelta
 from typing import Dict, Optional, Type, TypeVar
-from uuid import uuid4
+from uuid import UUID, uuid4
 
 from aiohttp import BasicAuth, ClientSession
 from aiohttp.client_exceptions import ClientError
@@ -14,31 +13,31 @@ from .system import System, SystemV2, SystemV3  # noqa
 
 _LOGGER = logging.getLogger(__name__)
 
-DEFAULT_USER_AGENT = "SimpliSafe/2105 CFNetwork/902.2 Darwin/17.7.0"
-DEFAULT_AUTH_USERNAME = "{0}.2074.0.0.com.simplisafe.mobile"
+DEFAULT_USER_AGENT: str = "SimpliSafe/2105 CFNetwork/902.2 Darwin/17.7.0"
+DEFAULT_AUTH_USERNAME: str = "{0}.2074.0.0.com.simplisafe.mobile"
 
-SYSTEM_MAP = {2: SystemV2, 3: SystemV3}
+SYSTEM_MAP: Dict[int, Type[System]] = {2: SystemV2, 3: SystemV3}
 
-URL_HOSTNAME = "api.simplisafe.com"
-URL_BASE = "https://{0}/v1".format(URL_HOSTNAME)
+URL_HOSTNAME: str = "api.simplisafe.com"
+URL_BASE: str = f"https://{URL_HOSTNAME}/v1"
 
 ApiType = TypeVar("ApiType", bound="API")
 
 
-class API:
+class API:  # pylint: disable=too-many-instance-attributes
     """Define an API object to interact with the SimpliSafe cloud."""
 
     def __init__(self, websession: ClientSession) -> None:
         """Initialize."""
-        self._access_token = ""
-        self._access_token_expire = None  # type: Optional[datetime]
-        self._actively_refreshing = False
-        self._email = None  # type: Optional[str]
-        self._refresh_token = ""
-        self._uuid = uuid4()
-        self._websession = websession
-        self.refresh_token_dirty = False
-        self.user_id = None
+        self._access_token: str = ""
+        self._access_token_expire: Optional[datetime] = None
+        self._actively_refreshing: bool = False
+        self._email: Optional[str] = None
+        self._refresh_token: str = ""
+        self._uuid: UUID = uuid4()
+        self._websession: ClientSession = websession
+        self.refresh_token_dirty: bool = False
+        self.user_id: Optional[str] = None
 
     @property
     def refresh_token(self) -> str:
@@ -82,7 +81,7 @@ class API:
 
     async def _authenticate(self, payload_data: dict) -> None:
         """Request token data and parse it."""
-        token_resp = await self.request(
+        token_resp: dict = await self.request(
             "post",
             "api/token",
             data=payload_data,
@@ -99,7 +98,7 @@ class API:
         )
         self.refresh_token = token_resp["refresh_token"]
 
-        auth_check_resp = await self.request("get", "api/authCheck")
+        auth_check_resp: dict = await self.request("get", "api/authCheck")
         self.user_id = auth_check_resp["userId"]
 
     async def _refresh_access_token(self, refresh_token: str) -> None:
@@ -116,9 +115,9 @@ class API:
 
     async def get_systems(self) -> Dict[str, System]:
         """Get systems associated to this account."""
-        subscription_resp = await self.get_subscription_data()
+        subscription_resp: dict = await self.get_subscription_data()
 
-        systems = {}
+        systems: Dict[str, System] = {}
         for system_data in subscription_resp["subscriptions"]:
             version = system_data["location"]["system"]["version"]
             system_class = SYSTEM_MAP[version]
@@ -130,10 +129,8 @@ class API:
 
     async def get_subscription_data(self) -> dict:
         """Get the latest location-level data."""
-        subscription_resp = await self.request(
-            "get",
-            "users/{0}/subscriptions".format(self.user_id),
-            params={"activeOnly": "true"},
+        subscription_resp: dict = await self.request(
+            "get", f"users/{self.user_id}/subscriptions", params={"activeOnly": "true"}
         )
 
         _LOGGER.debug("Subscription response: %s", subscription_resp)
@@ -149,7 +146,7 @@ class API:
         params: dict = None,
         data: dict = None,
         json: dict = None,
-        **kwargs
+        **kwargs,
     ) -> dict:
         """Make a request."""
         if (
@@ -160,12 +157,12 @@ class API:
             self._actively_refreshing = True
             await self._refresh_access_token(self._refresh_token)
 
-        url = "{0}/{1}".format(URL_BASE, endpoint)
+        url: str = f"{URL_BASE}/{endpoint}"
 
         if not headers:
             headers = {}
         if not kwargs.get("auth") and self._access_token:
-            headers["Authorization"] = "Bearer {0}".format(self._access_token)
+            headers["Authorization"] = f"Bearer {self._access_token}"
         headers.update({"Host": URL_HOSTNAME, "User-Agent": DEFAULT_USER_AGENT})
 
         try:
@@ -176,7 +173,7 @@ class API:
                 params=params,
                 data=data,
                 json=json,
-                **kwargs
+                **kwargs,
             ) as resp:
                 resp.raise_for_status()
                 return await resp.json(content_type=None)
@@ -195,7 +192,7 @@ class API:
                         params=params,
                         data=data,
                         json=json,
-                        **kwargs
+                        **kwargs,
                     )
                 raise InvalidCredentialsError
             if "403" in str(err):
@@ -204,6 +201,4 @@ class API:
                     return {}
                 raise InvalidCredentialsError
 
-            raise RequestError(
-                "Error requesting data from {0}: {1}".format(endpoint, err)
-            )
+            raise RequestError(f"Error requesting data from {endpoint}: {err}")
